const LSKeys = {
    SCRIPTS: 'claspo-scripts'
}

class Content {
    existsScriptNodes = [];
    cssSelector = 'claspo-lts-selector-class-for-control__'

    // <script>console.log(1)</script>

    constructor() {
    }

    addScript(scriptText) {
        this.existsScriptNodes = [];
        let _doc = new DOMParser().parseFromString(scriptText.trim(), 'text/html');

        _doc.head.childNodes.forEach((node) => {
            let _scriptText = document.createElement('script').textContent = node.textContent;
            const script = document.createElement('script');
            script.textContent = _scriptText;
            script.classList.add(this.cssSelector);
            document.body.append(script);
            this.existsScriptNodes.push(script);
        })
        localStorageService.setItem(LSKeys.SCRIPTS, scriptText);
    }

    deleteScript() {
        this.existsScriptNodes.forEach((node) => {
            node.remove();
        })

        this.existsScriptNodes = [];

        localStorageService.removeItem(LSKeys.SCRIPTS);
    }

    getScript() {
        const script = localStorageService.getItem(LSKeys.SCRIPTS);
        if (script) {
            this.addScript(script);
            chrome.runtime.sendMessage({type: "notification", options: {
                    type: "icon-state-checker",
                    state: true
                }});
        }
    }

    init() {
        this.getScript();
    }
}

class LocalStorageService {
    constructor() {
    }

    setItem(key, value) {
        localStorage.setItem(key, value);
    }

    getItem(key) {
        return localStorage.getItem(key);
    }

    removeItem(key) {
        localStorage.removeItem(key);
    }
}

const localStorageService = new LocalStorageService();
const content = new Content();

content.init();

chrome.runtime.onMessage.addListener(async (request, sender, response) => {
    if (request.todo === 'addScript') {
        content.addScript(request.scriptText);
        chrome.runtime.sendMessage({type: "notification", options: {
                type: "icon-state-checker",
                state: !!content.existsScriptNodes.length
            }});
    }

    if (request.todo === 'deleteScript') {
        content.deleteScript();
        chrome.runtime.sendMessage({type: "notification", options: {
                type: "icon-state-checker",
                state: !!content.existsScriptNodes.length
            }});
    }

    if ((request.from === 'popup') && (request.subject === 'scriptIsExist')) {
        let _response = '';
        content.existsScriptNodes.forEach((node) => {
            let scriptText = node.outerHTML.replace(`class="${content.cssSelector}"`, '');
            _response += scriptText;
        })
        response(_response);
    }

    if (request.method === 'icon-state-checker') {
        response(!!content.existsScriptNodes.length);
    }
});

window.addEventListener('storage', function(e) {
    if (e.key === null && e.oldValue === null && e.newValue === null && e.storageArea.length === 0) {
        content.deleteScript();
        chrome.runtime.sendMessage({type: "notification", options: {
                type: "icon-state-checker",
                state: false
            }});
    }
});
